#include <stdio.h>
#include <math.h>
/*

Programa para integrar numericamente la ecuacion diferencial
del pendulo ideal forzado y con rozamiento:

l * x'' + r * x' + G sin (x) =   f0 * cos ( ome * t);
      
donde l, r, G, f0, ome son parametros que se leen de un archivo
      llamado par_pend.txt 
      t es el tiempo
      x es la funcion del tiempo que se quiere calcular (x(t) es el
      angulo que forma el pendulo con la vertical)
      x'' indica la derivada segunda de x respecto del tiempo y x'
      la derivada primera
      Esta ecuacion necesita dos condiciones iniciales (x(t=0) y x'(t=0)=v_x(t=0))
      que se leen del archivo par_pend.txt
Vean que si eligen r=0 y f0 = 0 se trata de la ecuacion del pendulo ideal
sin forzado y sin rozamiento. 

En este programa se usa un metodo de integracion muy sencillo llamado
metodo de Euler. 

*/
/*------DARLE SU VALOR A LAS VARIABLES GLOBALES CON VALORES FIJOS--------------------*/

#define G (9.81)        /*  aceleracion de la gravedad */

/*------DEFINIR EL TIPO DE LAS VARIABLES GLOBALES-------------------------------*/

double l;                /* longitud del pendulo */
double dt;               /* paso de integracion   */
double r;                /* rozamiento */
double t_total;          /* tiempo total de integracion */
double ome;              /* frecuencia del forzado */
double f0;               /* amplitud del forzado por unidad de masa */
double F_x;              /* fuerza total por unidad de masa y longitud */
double x, v_x, t;

void leer_parametros ();
void rk4(double u[2], int n_var, double t);
void deri(int n_var,double u[2],double du[2],double t);

/*----------------FIN DEFINICIONES GLOBALES--------------------------------*/


/*------------------------------------------------------------------------------
  NAME:              main
  DESCRIPTION:       Programa principal. Donde empieza el programa
------------------------------------------------------------------------------*/

void
main ()
{
  double t;             /* tiempo actual                  */
  int n_pasos;          /* Numero de pasos de integracion */
  int i;                /* Variable temporal              */

  /* Leer parametros fisicos de un archivo que llamamos par_pend.txt    */
  leer_parametros ();

  /* Arrancar la simulacion                                               */
  t = 0.;

  /* Imprimir condicion inicial en la pantalla o en un archivo llamado "algo"
     si se corre direccionando la salida asi: prog1 > algo */
  printf ("%f   ", t);
  printf ("%f %f ", x, v_x);
  printf ("\n");

  /* Calcular cuantos pasos tenemos que hacer para integrar hasta t_total */
  n_pasos = t_total / dt;


  /* Integrar la ecuacion 'n_pasos' veces                                 */
  for ( i = 0; i < n_pasos; i = i + 1)
    {
      /* Calcular aceleracion angular */
      F_x = (-r * v_x - G * sin (x) + f0 * cos ( ome * t))/l;
      
      /* Calcular v_x(t+dt) */
      v_x = v_x + F_x * dt;
      
      /* Calcular x(t+dt) */
      x = x + v_x * dt;
      
      /* Actualizar el tiempo */
      t = t + dt;

      /* Imprimir los resultados en la pantalla o en un archivo llamado "algo"
       si se corre direccionando la salida asi: prog1 > algo */
      printf ("%f   ", t);
      printf ("%f %f ", x, v_x);
      printf ("\n");
      
      
    }
}


/*------------------------------------------------------------------------------
  NAME:              leer_parametros
  DESCRIPTION:       Lee los parametros de un archivo 'par_pend.txt'. El
                     formato es:
  #
  # comentarios
  # 
  l r f0 ome dt t_total
  x(t=0)
  v_x(t=0)

  ------------------------------------------------------------------------------*/

void
leer_parametros ()
{
  FILE *fp;
  char buffer[1024];

  fp = fopen ("par_pend.txt", "r");
  if (fp == NULL)
    {
      printf ("No pude abrir el archivo <par_pend.txt.\n");
      printf ("Programa abortado.\n");
      exit (-1);
    }

  while (1)
    {
      if (fgets (buffer,1024,fp) == NULL)
	break;
      if (buffer[0] == '#')
	continue;
      else
	break;
    }

  sscanf (buffer, "%lf %lf %lf %lf %lf %lf", &l, &r, &f0, &ome, &dt, 
	  &t_total);
  fscanf (fp, "%lf %lf", &x, &v_x);
}

