#include <stdio.h>
#include <math.h>
/*

Programa para integrar numericamente una ecuacion diferencial
de la forma:

m * x'' =  -k * x * x * x + a * x + f0 * cos ( ome * t);
      
donde m, k, a, f0, ome son parametros que se leen de un archivo
      llamado parametros.txt 
      t es el tiempo
      x es la funcion del tiempo que se quiere calcular (x(t) es la
      posicion de una particula de masa m sometida a una fuerza igual
      a la parte derecha de la ecuacion
      x'' indica la derivada segunda de x respecto del tiempo
      Esta ecuacion necesita dos condiciones iniciales (x(t=0) y x'(t=0)=v_x(t=0))
      que se leen del archivo parametros.txt
Vean que si eligen k = 0, a>0 y f0 = 0 se trata de la ecuacion del 
oscilador armonico.

En este programa se usa un metodo de integracion muy sencillo llamado
metodo de Euler. 

*/

/*-----------------------------DEFINIR VARIABLES-------------------------------*/

double k;                /* constante del resorte */
double m;                /* masa de la particula  */
double dt;               /* paso de integracion   */
double a;                /* otra constante para la fuerza */
double t_total;          /* tiempo total de integracion */
double ome;              /* frecuencia del forzado */
double f0;               /* amplitud del forzado */
double x, v_x;           /* posicion y velocidad */
double F_x;              /* fuerza total */

void leer_parametros ();

/*-----------------------------FIN DEFINICIONES--------------------------------*/

/*------------------------------------------------------------------------------
  NAME:              main
  DESCRIPTION:       Programa principal. Donde empieza el programa
------------------------------------------------------------------------------*/

void
main ()
{
  double t;             /* tiempo actual                  */
  int n_pasos;          /* Numero de pasos de integracion */
  int i;                /* Variable temporal              */

  /* Leer parametros fisicos de un archivo que llamamos parametros.txt    */
  leer_parametros ();

  /* Arrancar la simulacion                                               */
  t = 0;

  /* Imprimir condicion inicial en la pantalla o en un archivo llamado "algo"
     si se corre direccionando la salida asi: prog1 > algo */
  printf ("%f   ", t);
  printf ("%f %f ", x, v_x);
  printf ("\n");

  /* Calcular cuantos pasos tenemos que hacer para integrar hasta t_total */
  n_pasos = t_total / dt;


  /* Integrar la ecuacion 'n_pasos' veces                                 */
  for ( i = 0; i < n_pasos; i = i + 1)
    {
      /* Calcular fuerza */
      F_x = -k * x * x * x + a * x + f0 * cos ( ome * t);
      
      /* Calcular v(t+dt) */
      v_x = v_x + F_x / m * dt;
      
      /* Calcular x(t+dt) */
      x = x + v_x * dt;
      
      /* Actualizar el tiempo */
      t = t + dt;

      /* Imprimir los resultados en la pantalla o en un archivo llamado "algo"
       si se corre direccionando la salida asi: prog1 > algo */
      printf ("%f   ", t);
      printf ("%f %f ", x, v_x);
      printf ("\n");
      
      
    }
}



/*------------------------------------------------------------------------------
  NAME:              leer_parametros
  DESCRIPTION:       Lee los parametros de un archivo 'parametros.txt'. El
                     formato es:
  #
  # comentarios
  # 
  k m a f0 ome dt t_total
  x   (t=0)
  v_x (t=0)

  ------------------------------------------------------------------------------*/

void
leer_parametros ()
{
  FILE *fp;
  char buffer[1024];

  fp = fopen ("parametros.txt", "r");
  if (fp == NULL)
    {
      printf ("No pude abrir el archivo <parametros.txt.\n");
      printf ("Programa abortado.\n");
      exit (-1);
    }

  while (1)
    {
      if (fgets (buffer,1024,fp) == NULL)
	break;
      if (buffer[0] == '#')
	continue;
      else
	break;
    }

  sscanf (buffer, "%lf %lf %lf %lf %lf %lf %lf", &k, &m, &a, &f0, &ome, &dt, 
	  &t_total);
  fscanf (fp, "%lf %lf", &x, &v_x);
}
